APP_NAME = "intra-search"
STATIC_FOLDER = "../ui/dist"
DEFAULT_PORT = 5000
DEFAULT_CHUNK_SIZE = 50
DEFAULT_MODEL = "msmarco-distilbert-cos-v5"
HOST = "localhost"
SHORT_DESC = "A tool for performing semantic search within documents using sentence transformers to find contextually relevant text."
